@@ -9,7 +9,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: embperl.c 10075 2004-11-25 13:40:40Z dfs $";
+"$Id: embperl.c 12279 2006-08-30 18:11:14Z dfs $";
 
 #ifdef EMBED_PERL
 #include <EXTERN.h>
@@ -20,26 +20,6 @@ static char const RCSID[] =
 static PerlInterpreter *my_perl = NULL;
 extern void xs_init ();
 
-/**********************************************************************
-* %FUNCTION: num_open_descriptors
-* %ARGUMENTS:
-*  None
-* %RETURNS:
-*  An estimate of the number of open file descriptors.
-* %DESCRIPTION:
-*  This is a truly evil function.
-***********************************************************************/
-static int
-num_open_descriptors(void)
-{
-    int i, total;
-    total = 0;
-    for (i=0; i<1024; i++) {
-	if (fcntl(i, F_GETFL, 0) != -1) total++;
-    }
-    return total;
-}
-
 int
 make_embedded_interpreter(char const *progPath,
 			  char const *subFilter,
@@ -47,7 +27,6 @@ make_embedded_interpreter(char const *progPath,
 {
     char *argv[6];
     int argc;
-    int descriptors;
 
     if (my_perl != NULL) {
 #ifdef SAFE_EMBED_PERL
@@ -95,12 +74,8 @@ make_embedded_interpreter(char const *progPath,
 	argv[3] = NULL;
 	argc = 3;
     }
-    descriptors = num_open_descriptors();
     perl_parse(my_perl, xs_init, argc, argv, NULL);
     perl_run(my_perl);
-    if (num_open_descriptors() > descriptors) {
-	syslog(LOG_INFO, "WARNING: Something in your Perl filter appears to have opened a file descriptor outside of any function.  With embedded Perl, you should move any code that opens a file descriptor into filter_initialize.  On some systems, the C library may open a descriptor, but you should verify your filter just in case.");
-    }
     return 0;
 }
 
