@@ -16,7 +16,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: mimedefang.c 13974 2007-02-09 16:29:23Z dfs $";
+"$Id: mimedefang.c 14671 2007-04-02 17:33:57Z dfs $";
 
 #define _BSD_SOURCE 1
 
@@ -66,6 +66,7 @@ typedef int md_socklen_t;
 #include <netdb.h>
 
 #include "libmilter/mfapi.h"
+#include "milter_cap.h"
 
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -116,6 +117,10 @@ int ConserveDescriptors = 0;
 /* Log "eom" run-times */
 int LogTimes = 0;
 
+/* Show me rejected recipients?
+   (Sendmail 8.14.0 and higher only */
+int SeeRejectedRecipients = 1;
+
 /* Default Backlog for "listen" */
 static int Backlog = -1;
 
@@ -933,6 +938,77 @@ envfrom(SMFICTX *ctx, char **from)
     return SMFIS_CONTINUE;
 }
 
+/**********************************************************************
+*%FUNCTION: mf_data
+*%ARGUMENTS:
+* ctx -- Sendmail filter mail context
+*%RETURNS:
+* Standard milter reply code
+*%DESCRIPTION:
+* Does a post-DATA callback
+***********************************************************************/
+#ifdef MILTER_BUILDLIB_HAS_DATA
+static sfsistat mf_data(SMFICTX *ctx)
+{
+    return SMFIS_CONTINUE;
+}
+#endif
+
+/**********************************************************************
+*%FUNCTION: mf_unknown
+*%ARGUMENTS:
+* ctx -- Sendmail filter mail context
+*%RETURNS:
+* Standard milter reply code
+*%DESCRIPTION:
+* Does a post-DATA callback
+***********************************************************************/
+#ifdef MILTER_BUILDLIB_HAS_UNKNOWN
+static sfsistat mf_unknown(SMFICTX *ctx, char const *arg)
+{
+    return SMFIS_CONTINUE;
+}
+#endif
+
+/**********************************************************************
+*%FUNCTION: mf_negotiate
+*%ARGUMENTS:
+* ctx -- Sendmail filter mail context
+* Remaining args -- crappy args as described in libmilter docs.
+*%RETURNS:
+* Standard milter reply code
+*%DESCRIPTION:
+* Negotiates protocol options
+***********************************************************************/
+#ifdef MILTER_BUILDLIB_HAS_NEGOTIATE
+static sfsistat mf_negotiate(SMFICTX *ctx,
+			     unsigned long f0,
+			     unsigned long f1,
+			     unsigned long f2,
+			     unsigned long f3,
+			     unsigned long *pf0,
+			     unsigned long *pf1,
+			     unsigned long *pf2,
+			     unsigned long *pf3)
+{
+    *pf0 = f0;
+    *pf1 = 0;
+    *pf2 = 0;
+    *pf3 = 0;
+
+    if (f1 & SMFIP_RCPT_REJ) {
+	if (SeeRejectedRecipients) {
+	    *pf1 |= SMFIP_RCPT_REJ;
+	}
+    }
+    /* Don't want leading spaces */
+    *pf1 &= (~SMFIP_HDR_LEADSPC);
+
+    return SMFIS_CONTINUE;
+
+}
+#endif
+
 /**********************************************************************
 *%FUNCTION: rcptto
 *%ARGUMENTS:
@@ -1528,7 +1604,7 @@ eom(SMFICTX *ctx)
 	/* Check line length */
 	if (eptr - rptr >= SMALLBUF-1) {
 	    syslog(LOG_WARNING, "%s: Overlong line in RESULTS file - %d chars (max %d)",
-		   data->qid, eptr - rptr, SMALLBUF-1);
+		   data->qid, (int) (eptr - rptr), SMALLBUF-1);
 	    cleanup(ctx);
 	    r = SMFIS_TEMPFAIL;
 	    goto bail_out;
@@ -1690,6 +1766,17 @@ eom(SMFICTX *ctx)
 	    MD_SMFI_TRY(do_sm_quarantine, (ctx, rptr+1));
 	    break;
 
+	case 'f':
+	    /* Change the "from" address */
+#ifdef MILTER_BUILDLIB_HAS_CHGFROM
+	    if (!rejecting) {
+		percent_decode(rptr+1);
+		MD_SMFI_TRY(smfi_chgfrom, (ctx, rptr+1, NULL));
+	    }
+#else
+	    syslog(LOG_WARNING, "change_sender called, but this version of libmilter does not support CHGFROM");
+#endif
+	    break;
 	case 'S':
 	    /* Delete a recipient */
 	    if (!rejecting) {
@@ -1884,17 +1971,15 @@ static struct smfiDesc filterDescriptor =
     "MIMEDefang-" VERSION,      /* Filter name */
     SMFI_VERSION,		/* Version code */
 
-#if SMFI_VERSION >= 0x01000000
-    SMFIF_ADDHDRS|SMFIF_CHGBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT|SMFIF_CHGHDRS|SMFIF_QUARANTINE,
-#elif SMFI_VERSION == 2
+#if SMFI_VERSION >= 2
+    SMFIF_ADDHDRS|SMFIF_CHGBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT|SMFIF_CHGHDRS
 #ifdef SMFIF_QUARANTINE
-    /* We can: quarantine, add a header and may alter body
-       and add/delete recipients*/
-    SMFIF_ADDHDRS|SMFIF_CHGBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT|SMFIF_CHGHDRS|SMFIF_QUARANTINE,
-#else
-    /* We can: Add headers, alter body, add/delete recipients, alter headers */
-    SMFIF_ADDHDRS|SMFIF_CHGBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT|SMFIF_CHGHDRS,
+    |SMFIF_QUARANTINE
 #endif
+#ifdef MILTER_BUILDLIB_HAS_CHGFROM
+    |SMFIF_CHGFROM
+#endif
+    ,
 #elif SMFI_VERSION == 1
     /* We can: add a header and may alter body and add/delete recipients*/
     SMFIF_MODHDRS|SMFIF_MODBODY|SMFIF_ADDRCPT|SMFIF_DELRCPT,
@@ -1910,11 +1995,17 @@ static struct smfiDesc filterDescriptor =
     eom,			/* Called at end of message */
     mfabort,			/* Called on abort */
     mfclose			/* Called on connection close */
-#if SMFI_VERSION >= 0x01000000
+#ifdef MILTER_BUILDLIB_HAS_UNKNOWN
     ,
-    NULL,                       /* xxfi_unknown */
-    NULL,                       /* xxfi_data    */
-    NULL                        /* xxfi_negotiate */
+    mf_unknown			/* xxfi_unknown */
+#endif
+#ifdef MILTER_BUILDLIB_HAS_DATA
+    ,
+    mf_data			/* xxfi_data    */
+#endif
+#ifdef MILTER_BUILDLIB_HAS_NEGOTIATE
+    ,
+    mf_negotiate                /* xxfi_negotiate */
 #endif
 };
 
@@ -1959,6 +2050,9 @@ usage(void)
     fprintf(stderr, "  -H                -- Do HELO checks before processing any messages\n");
     fprintf(stderr, "  -c                -- Strip bare <CR> characters from message body\n");
     fprintf(stderr, "  -R num            -- Reserve num slaves for connections from localhost\n");
+    fprintf(stderr, "  -N                -- Do not pass rejected recipients to milter (Sendmail\n");
+    fprintf(stderr, "                       8.14.0 and newer only.)\n");
+
     exit(EXIT_FAILURE);
 }
 
@@ -1990,6 +2084,12 @@ main(int argc, char **argv)
     void *ctx = NULL;
 #endif
 
+    /* If first arg is "prcap", just print milter capabilities and quit */
+    if (argc == 2 && !strcmp(argv[1], "prcap")) {
+	dump_milter_buildlib_info();
+	exit(0);
+    }
+
     /* Paranoia time */
     umask(027);
 
@@ -2032,8 +2132,15 @@ main(int argc, char **argv)
     }
 
     /* Process command line options */
-    while ((c = getopt(argc, argv, "CDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:")) != -1) {
+    while ((c = getopt(argc, argv, "NCDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:")) != -1) {
 	switch (c) {
+	case 'N':
+#ifdef MILTER_BUILDLIB_HAS_NEGOTIATE
+	    SeeRejectedRecipients = 0;
+#else
+	    fprintf(stderr, "-N option only available with Sendmail/Milter 8.14.0 and higher... ignoring\n");
+#endif
+	    break;
 	case 'R':
 	    sscanf(optarg, "%d", &slavesReservedForLoopback);
 	    if (slavesReservedForLoopback < -1) {
